@@ -20,7 +20,8 @@ DESCRIPTION
 
         <<SeeSaw(section="section", toshow="toshow", tohide="tohide",
                  show=True|False, bg="background", inline="inline",
-                 image="arrow|plumin", speed="slow|normal|fast"|integer,
+                 image="arrow|plumin|attachment|file",
+                 speed="slow|normal|fast"|integer,
                  seesaw=True|False, addclass="list", type="link|button",
                  effect="slide|fade")>>
     where
@@ -31,26 +32,25 @@ DESCRIPTION
                 name, and needs to be attached to sections that the control is
                 to affect; the default is 'section'
                  
-         toshow specifies the text to display for showing the section; 
+         toshow specifies the text or image to display for showing the section; 
                 the default is 'Show', except for inline sections where it
                 is '»»'
                  
-         tohide specifies the text to display for hiding the section;
+         tohide specifies the text or image to display for hiding the section;
                 the default is 'Hide', except for inline sections where it
                 is '««'
                 
            show specifies whether the section is to be shown initially;
                 the default is False
                  
-             bg specifies the a background colour to be applied to the section;
+             bg specifies a background colour to be applied to the section;
                 the default is None
                  
          inline specifies the text of an inline section; SeeSaw creates inline
                 sections itself, immediately after the controls;
                 the default is None
                  
-          image selects an image set to use for the control instead of text,
-                with toshow and tohide being ignored; 
+          image specifies the source of a pair of images to be used by the control;
                 the default is None;
                 
           speed specifies the rate at which the section should appear or 
@@ -154,15 +154,26 @@ EXTENDED DESCRIPTION
     matching SeeSaw call if its 'tohide' and 'toshow' arguments are different
     (so that the correct one can be shown initially).
 
-    'toshow' and 'tohide' can accommodate text surrounding the control, to
-    allow it to be different in the two cases. The three parts are
-    distinguished by enclosing the text of the control between '<<' and '>>',
-    e.g. toshow="<<Open>> me up", tohide="Now <<close>> me down". The middle
-    part is ignored if 'image' is used, so can be empty.
-
     The toshow and tohide defaults only apply if both options are not set. If
     only one is set the other is given its value.
-                 
+
+    'toshow' and 'tohide' can accommodate text surrounding the control, to
+    allow it to be different in the two cases. The three parts are
+    distinguished by enclosing the text or image of the control between '<<'
+    and '>>', e.g. toshow="<<Open>> me up", tohide="Now <<close>> me down". 
+
+    (The middle parts of) 'toshow' and 'tohide' can be set independently to be
+    images using the following notation:
+
+        <<SeeSaw(s, Show, attachment:hide.png )>>
+
+    to use an image from the page's attachments directory, and
+
+        <<SeeSaw(s, file:/path/to/show.png, Hide)>>
+
+    where /path/to/hide.png is a server path for loading the image from the
+    filesystem.
+
  bg
     If a background colour is specified for a section, '"section"-bg'
     needs to be added to the corresponding markup to have the colour
@@ -177,7 +188,6 @@ EXTENDED DESCRIPTION
     precedence if multiple, but different, values are given.
 
  inline
-
     The text of inline sections is embedded in SeeSaw calls. By default, inline
     sections are hidden initially, and the effects of 'show' and 'bg', and the
     addition of the 'section' and 'addclass' classes are handled directly by
@@ -187,10 +197,26 @@ EXTENDED DESCRIPTION
 
  image
     Additional image sets can be made available as embedded data or from files.
-    Instructions are in the 'Configuration' section of this file (below).
+    Instructions are in the 'Configuration' section of this file (below). When
+    'image' specifies one of the provided sets, or one added by the installer,
+    the middle parts of 'toshow' and 'tohide' are ignored.
+
+    When 'image' has the value 'attachment', 'toshow' and 'tohide' (strictly
+    the middle parts) are interpreted as the names of images in the page's
+    attachments directory, e.g.
+
+        <<SeeSaws(s, show.png, hide.png, image=attachment)>>
+
+    Similarly, when 'image' has the value 'file', 'toshow' and 'tohide' are
+    interpreted as server paths for directly loading user-specified images from
+    the filesystem, e.g.
+
+        <<SeeSaws(s, image=file, toshow=/img/show.png, tohide=/img/hide.png)>>
+
+    Note that if 'image=' is used out of order, 'toshow=' and 'tohide=' must
+    be used.
 
  seesaw 
- 
     By setting the option 'seesaw' to False it is possible to apply SeeSaw to
     non-SeeSaw sections, e.g. the div created by the TableOfContents macro. In
     such a case, the section name in the SeeSaw call should be a class already
@@ -237,6 +263,12 @@ AUTHOR
     Jim Wight <jkwight@gmail.com>
 
 HISTORY
+    [v1.5] 2013-06-09
+    Add attachment and file options to image
+    Support attachment: and file: prefixes in toshow and tohide for images
+    Make <<>> in toshow and tohide apply to images
+    Modify first call detection according to MoinMoin version
+
     [v1.4] 2013-04-28
     Support both execute (Moin 1.6) and macro_MacroName (1.7+) interfaces
     Make loading of JavaScript configurable
@@ -280,9 +312,9 @@ HISTORY
 
 #--- Configuration --------------------------------------------
 # 
-jqueryjs = 'http://code.jquery.com/jquery-1.9.1.min.js'      # No installation required
+jqueryjs = 'http://code.jquery.com/jquery-1.10.1.min.js'     # No installation required
 # jqueryjs = '/jscript/jquery.js'                            # Or similar to load from a local file 
-# jqueryjs = None                                            # If jquery being loaded via html_head
+# jqueryjs = None                                            # If jQuery being loaded via html_head
 #                                                            # See INSTALL.html
 #
 seesawjs = ''                                                # No installation required
@@ -313,13 +345,13 @@ Dependencies = []
 pagename = ''
 
 # MoinMoin 1.7 and later interface
-def macro_SeeSaw(macro, section=u'section', toshow=unicode, tohide=unicode, show=False, bg=unicode, inline=unicode, image=unicode, speed=u'0', seesaw=True, addclass=u'', type=(u'link',u'button'), effect=(u'dflt',u'slide', u'fade')):
+def macro_SeeSaw(macro, section=u'section', toshow=unicode, tohide=unicode, show=False, bg=u'', inline=u'', image=u'', speed=u'0', seesaw=True, addclass=u'', type=(u'link',u'button'), effect=(u'dflt',u'slide',u'fade')):
     # Compensate for lack of single-quoted argument support
     def squote(item):
         if item in (None, True, False): return item
         return item.lstrip("'").rstrip("'")
 
-    html = SeeSaw(macro, map(squote,(section,toshow,tohide,show,bg,inline,image,speed,seesaw,addclass,type,effect))) 
+    html = SeeSaw(macro, map(squote, (section, toshow, tohide, show, bg, inline, image, speed, seesaw, addclass, type, effect))) 
 
     return macro.formatter.rawHTML(html)
 
@@ -332,13 +364,31 @@ def execute(macro, args):
 
     if section is None:
         section = 'section'
+
+    if show is None:
+        show = False
         
+    if bgnd is None:
+        bgnd = ''
+
+    if inline is None:
+        inline = ''
+        
+    if image is None:
+        image = ''
+
     if speed is None:
         speed = 0
 
+    if seesaw is None:
+        seesaw = True
+
     if addclass is None:
         addclass = ""
 
+    if control is None:
+        control = 'link'
+
     if effect not in ['slide', 'fade']:
         effect = 'dflt'
 
@@ -349,14 +399,9 @@ def execute(macro, args):
 
 def SeeSaw(macro, (section, toshow, tohide, show, bgnd, inline, image, speed, seesaw, addclass, control, effect)):
     import re
+    from MoinMoin import version
     global pagename
 
-    def escape(arg):
-        if arg is None:
-            return arg
-        else:
-            return wikiutil.escape(arg)
-
     images = dict(
 arrow = [
 '''data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAkAAAAJCAYAAADgkQYQAAAAU0lEQVQYlZ2QIQ6AQAwEJ9lcU4mt
@@ -376,7 +421,7 @@ phCliIWAtYzoihgpso4oRYw43IICAElFBRBz9sLvAAAAAElFTkSuQmCC
     images.update(moreimages)
 
     if tohide is None and toshow is None:
-        if inline is None:
+        if not inline:
             toshow = 'Show'; tohide = 'Hide'
         else:
             toshow = u'»»';  tohide = u'««'
@@ -385,13 +430,6 @@ phCliIWAtYzoihgpso4oRYw43IICAElFBRBz9sLvAAAAAElFTkSuQmCC
     elif toshow is None and tohide is not None:
         toshow = tohide
 
-    if image:
-        imagepair = images.get(image)
-        if imagepair:
-            if not imagepair[0].startswith("data:image"):
-                imagepair = (imageprefix.rstrip("/") + "/" + x for x in imagepair)
-            (toshow, tohide) = ('''<img src="%s" style="vertical-align:middle"/>''' % x for x in imagepair)
-
     try:
         speed = int(speed)
     except ValueError:
@@ -429,9 +467,35 @@ phCliIWAtYzoihgpso4oRYw43IICAElFBRBz9sLvAAAAAElFTkSuQmCC
     if matches:
         prehd, tohide, posthd = matches.groups()
 
-    section, preshw, postshw, prehd, posthd, bgnd, inline = (escape(x) for x in (section, preshw, postshw, prehd, posthd, bgnd, inline))
-    if not image:
-        toshow = escape(toshow); tohide = escape(tohide)
+    vermin = int(version.release_short[1])
+    if vermin < 9:
+        url = macro.request.getScriptname() + macro.request.getPathinfo()
+    else:
+        url = macro.request.script_root + macro.request.path
+
+    attimg = '''<img src="''' + wikiutil.escape(url + '''?action=AttachFile&do=get&target=%s''') + '''" style="vertical-align:middle"/>'''
+    fsimg = '''<img src="''' + wikiutil.escape("%s") + '''" style="vertical-align:middle"/>'''
+    if image:
+        if image == 'attachment':
+            toshow, tohide = (attimg % x for x in (toshow, tohide))
+        elif image == 'file':
+            toshow, tohide = (fsimg % x for x in (toshow, tohide))
+        else:
+            imagepair = images.get(image)
+            if imagepair:
+                if not imagepair[0].startswith("data:image"):
+                    imagepair = (imageprefix.rstrip("/") + "/" + x for x in imagepair)
+                toshow, tohide = (fsimg % x for x in imagepair)
+    else:
+        def toshowhide(arg):
+            if arg.startswith('attachment:'):
+                return attimg % arg[11:]
+            if arg.startswith('file:'):
+                return fsimg % arg[5:]
+            return wikiutil.escape(arg)
+        toshow, tohide = toshowhide(toshow), toshowhide(tohide)
+
+    section, preshw, postshw, prehd, posthd, bgnd, inline = (wikiutil.escape(x) for x in (section, preshw, postshw, prehd, posthd, bgnd, inline))
 
     if not inline:
         inlinesection = ""
@@ -483,9 +547,13 @@ phCliIWAtYzoihgpso4oRYw43IICAElFBRBz9sLvAAAAAElFTkSuQmCC
 %(inlinesection)s
 ''' % locals()
 
-    if  macro.formatter.page.page_name != pagename:
-        pagename = macro.formatter.page.page_name
-        html = SeeSawJS(jqueryjs,seesawjs) + html
+    if vermin < 9:
+        if  macro.formatter.page.page_name != pagename:
+            pagename = macro.formatter.page.page_name
+            html = SeeSawJS(jqueryjs, seesawjs) + html
+    else:
+        if  macro.request.uid_generator('SeeSaw') == 'SeeSaw':
+            html = SeeSawJS(jqueryjs, seesawjs) + html
 
     return html
 
@@ -505,8 +573,7 @@ td.seesaw.show { display: table-cell; }
     if seesaw is None:
         if jquery is None:
             return antijump
-	else:	   
-	    return '<p><span style="font-size:150%;color:red">Incompatible configuration combination of jquery and seesaw</p>'
+        return '<p><span style="font-size:150%;color:red">Incompatible configuration combination of jquery and seesaw</span></p>'
 
     if jquery is None:
         jqueryjs = ''
